require "json"

HOST_URL                   = "inv.nadeko.cl"
RESPONSE_HEADERS_BLACKLIST = {"access-control-allow-origin", "alt-svc", "server"}
REQUEST_HEADERS_WHITELIST  = {"accept", "accept-encoding", "cache-control", "content-length", "if-none-match", "range"}

module Routing
  extend self

  def self.proxy_file(response, env)
    if response.headers.includes_word?("Content-Encoding", "gzip")
      Compress::Gzip::Writer.open(env.response) do |deflate|
        IO.copy response.body_io, deflate
      end
    elsif response.headers.includes_word?("Content-Encoding", "deflate")
      Compress::Deflate::Writer.open(env.response) do |deflate|
        IO.copy response.body_io, deflate
      end
    else
      IO.copy response.body_io, env.response
    end
  end

  private def self.proxy_image(env, response)
    env.response.status_code = response.status_code
    response.headers.each do |key, value|
      if !RESPONSE_HEADERS_BLACKLIST.includes?(key.downcase)
        env.response.headers[key] = value
      end
    end

    env.response.headers["Access-Control-Allow-Origin"] = "*"

    if response.status_code >= 300
      return env.response.headers.delete("Transfer-Encoding")
    end

    return proxy_file(response, env)
  end

  def register_all
    get "/vi/:id/:name" do |env|
      thumbnails(env)
    end

    get "/debug" do |env|
      debug(env)
    end

    get "/debug2" do |env|
      debug(env)
    end

    get "/test" do |env|
      "test"
    end
  end

  def self.debug(env)
    env.response.content_type = "application/json"
    meow2 = [] of String
    pool_info = YTIMG_POOLS["i"].get_pool.stats

    pp pool_info

    xd = JSON.build do |z|
      z.object do
        z.field "pool_info" do
          z.object do
            z.field "pool_idle_conn", pool_info.idle_connections
            z.field "pool_max_conn", pool_info.in_flight_connections
            z.field "pool_in_fligth_conn", pool_info.max_connections
            z.field "pool_open_conn", pool_info.open_connections
          end
        end
        z.field "pool_capacity", YTIMG_POOLS["i"].inspect
      end
    end

    return xd
  end

  XD_CLIENT = HTTP::Client.new("i.ytimg.com")

  def self.thumbnails(env)
    id = env.params.url["id"]
    name = env.params.url["name"]

    headers = HTTP::Headers.new

    xd = {
      {host: HOST_URL, height: 720, width: 1280, name: "maxres", url: "maxres"},
      {host: HOST_URL, height: 720, width: 1280, name: "maxresdefault", url: "maxresdefault"},
      {host: HOST_URL, height: 480, width: 640, name: "sddefault", url: "sddefault"},
      {host: HOST_URL, height: 360, width: 480, name: "high", url: "hqdefault"},
      {host: HOST_URL, height: 180, width: 320, name: "medium", url: "mqdefault"},
      {host: HOST_URL, height: 90, width: 120, name: "default", url: "default"},
      {host: HOST_URL, height: 90, width: 120, name: "start", url: "1"},
      {host: HOST_URL, height: 90, width: 120, name: "middle", url: "2"},
      {host: HOST_URL, height: 90, width: 120, name: "end", url: "3"},
    }

    # if name == "maxres.jpg"
    #   xd.each do |thumb|
    #     thumbnail_resource_path = "/vi/#{id}/#{thumb[:url]}.jpg"
    #     if get_ytimg_pool("i").client &.head(thumbnail_resource_path, headers).status_code == 200
    #       name = thumb[:url] + ".jpg"
    #       break
    #     end
    #   end
    # end

    url = "/vi/#{id}/#{name}"

    REQUEST_HEADERS_WHITELIST.each do |header|
      if env.request.headers[header]?
        headers[header] = env.request.headers[header]
      end
    end

    begin
      # meow = HTTP::Client.new(url)

      # XD_CLIENT.get(url, headers: headers) do |resp|
      #   return self.proxy_image(env, resp)
      # end
      get_ytimg_pool("i").client &.get(url, headers) do |resp|
        return self.proxy_image(env, resp)
      end
    rescue ex
      puts "#{ex.message} + #{ex.inspect}"
    end
  end
end
